/**************************************************************************
**
** Copyright (C) Computer Science Department, Worcester Polytechnic Institute,
**                           All rights reserved.
**
** FILE NAME: RealTrackerView.cpp
**
** DESCRIPTION: This program implements the methods of CRealTrackerView class
**              
**
** AUTHOR: Zheng Zuo/Yubing Wang       March-2001
**
/**************************************************************************/


// RealTrackerView.cpp : implementation of the CRealTrackerView class
//

#include "stdafx.h"
#include "RealTracker.h"

#include "RealTrackerDoc.h"
#include "RealTrackerView.h"

#include "ControlOptionDlg.h"
#include "CheckInputDlg.h"
#include "ResultDeliveryDlg.h"
#include "StatsDlg.h"
#include "SaveLogDlg.h"

#include "VQTest.h"
#include "VQData.h"
#include "SysStatus.h"

#include "TimeStatusBar.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

void MakeRandomList(int order[], int n);

/////////////////////////////////////////////////////////////////////////////
// CRealTrackerView

IMPLEMENT_DYNCREATE(CRealTrackerView, CListView)

BEGIN_MESSAGE_MAP(CRealTrackerView, CListView)
	//{{AFX_MSG_MAP(CRealTrackerView)
	ON_WM_TIMER()
	ON_COMMAND(ID_CONTROL_PLAY, OnControlPlay)
	ON_COMMAND(ID_CONTROL_STOP, OnControlStop)
	ON_COMMAND(ID_CONTROL_OPTION, OnControlOption)
	ON_UPDATE_COMMAND_UI(ID_CONTROL_PLAY, OnUpdateControlPlay)
	ON_UPDATE_COMMAND_UI(ID_CONTROL_STOP, OnUpdateControlStop)
	ON_COMMAND(ID_FILE_USER, OnFileUser)
	ON_COMMAND(ID_VIEW_STAT, OnViewStat)
	ON_UPDATE_COMMAND_UI(ID_VIEW_STAT, OnUpdateViewStat)
    ON_MESSAGE(WM_VQACTION, OnVQAction)
	ON_COMMAND(ID_FILE_SAVELOG, OnFileSavelog)
	//}}AFX_MSG_MAP
    ON_UPDATE_COMMAND_UI(ID_INDICATOR_PROGRESS, OnUpdateProgress)
    ON_UPDATE_COMMAND_UI(ID_INDICATOR_TIMELINE, OnUpdateTimeLine)
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CRealTrackerView construction/destruction

CRealTrackerView::CRealTrackerView()
{
    m_bInitialized = FALSE;

    m_StatsDlg = NULL;
    m_StatsDlgIsVisable = FALSE;

    m_VQTest = NULL;
    m_ProgDspt = NULL;
    m_ProgList = NULL;

    m_SizeRatio = 1.0;

    m_Pref.IsFirstRun = FALSE;
    m_Pref.UserName = "";
    m_Pref.EmailAddr = "";
    m_Pref.CPUType = CSysStatus::GetCpuType();
    m_Pref.MEMCap = CSysStatus::GetMemory();
    m_Pref.OSType = CSysStatus::GetOSType();
    m_Pref.UserCpu = -1;
    m_Pref.NetConfig = -1;
    m_Pref.PlayTime = PLAYTIME_ONE;
    m_Pref.CheckQuality = CHECKQUALITY_BYCHOICE;
    m_Pref.LogFile = "";

    m_bSaveToFile = FALSE;
    m_SaveToDir = "c:\\";

    m_bPlaying = FALSE;
    m_bCurRunning = FALSE;
    m_bNextRunning = FALSE;

    m_ProgressLine = 0;
    m_TimeLine = "";
}

CRealTrackerView::~CRealTrackerView()
{
    if (m_StatsDlg != NULL)
    {
        m_StatsDlg->DestroyWindow();
        delete m_StatsDlg;
    }

    if (m_VQTest != NULL)
        delete m_VQTest;
    if (m_ProgDspt != NULL)
        delete [] m_ProgDspt;
    if (m_ProgList != NULL)
        delete [] m_ProgList;
}

BOOL CRealTrackerView::PreCreateWindow(CREATESTRUCT& cs)
{
    cs.style |= LVS_SHOWSELALWAYS | LVS_REPORT | LVS_SINGLESEL;

	return CListView::PreCreateWindow(cs);
}

/////////////////////////////////////////////////////////////////////////////
// CRealTrackerView drawing

void CRealTrackerView::OnDraw(CDC* pDC)
{
	CRealTrackerDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);
	// TODO: add draw code for native data here
}

/////////////////////////////////////////////////////////////////////////////
// CRealTrackerView diagnostics

#ifdef _DEBUG
void CRealTrackerView::AssertValid() const
{
	CListView::AssertValid();
}

void CRealTrackerView::Dump(CDumpContext& dc) const
{
	CListView::Dump(dc);
}

CRealTrackerDoc* CRealTrackerView::GetDocument() // non-debug version is inline
{
	ASSERT(m_pDocument->IsKindOf(RUNTIME_CLASS(CRealTrackerDoc)));
	return (CRealTrackerDoc*)m_pDocument;
}
#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// CRealTrackerView Initialization

void CRealTrackerView::OnInitialUpdate()
{
	CListView::OnInitialUpdate();

	CListCtrl& theListCtrl = GetListCtrl();

    if (! m_bInitialized)
    {
        m_bInitialized = TRUE;

        AdvancedPreference advPref;
        if (GetDocument()->ReadAutoConfig(advPref))
            m_AdvPref = advPref;
	        
        //((CMainFrame*)AfxGetMainWnd())->m_pListView = this;
        if (m_VQTest == NULL)
            m_VQTest = new CVQTest(this);

       theListCtrl.SetRedraw(FALSE); 
	    
	    //For the style LVS_EX_FULLROWSELECT, you'll need the following, depending on the OS:
	    //Windows NT: Requires version 5.0 or later (or version 4.0 with Internet Explorer 3.0 and later).. 
	    //Windows: Requires Windows 98 (or Windows 95 with Internet Explorer 3.0 or later). 
	    //Otherwise, you'll need to do ownerdrawn stuff. In that case, see the MFC ROWLIST sample.
	    DWORD dwExStyle = LVS_EX_FULLROWSELECT;
	    theListCtrl.SendMessage(LVM_SETEXTENDEDLISTVIEWSTYLE, (WPARAM)dwExStyle, (LPARAM)dwExStyle);

        COLORREF crYellow = RGB(240, 240, 200); //light yellow
	    theListCtrl.SetBkColor(crYellow);
	    theListCtrl.SetTextBkColor(crYellow);

	    theListCtrl.InsertColumn(0, "Name");
	    theListCtrl.InsertColumn(1, "URL");
        theListCtrl.SetColumnWidth(0, 150);
        theListCtrl.SetColumnWidth(1, 400);

	    theListCtrl.SetRedraw(TRUE);
	    theListCtrl.Invalidate(); 
    }
    else
    {
        theListCtrl.DeleteAllItems();
        if (m_ProgDspt != NULL)
            delete [] m_ProgDspt;
        if (m_ProgList != NULL)
            delete [] m_ProgList;
    }
    
//    m_nProgListNum = sizeof(progList) / sizeof(ProgListType);
	CRealTrackerDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);
	m_nProgListNum = pDoc->GetProgListNum();

    if (m_nProgListNum > 0)
    {
        int *order = new int [m_nProgListNum];
        MakeRandomList(order, m_nProgListNum);
        m_ProgDspt = new CString[m_nProgListNum];
        m_ProgList = new CString[m_nProgListNum];
        CString szDescript;
        for (int i=0; i<m_nProgListNum; i++)
        {
            //m_ProgDspt[i] = progList[order[i]].ProgDspt;
            //m_ProgList[i] = progList[order[i]].ProgUrl;
		    m_ProgDspt[i] = pDoc->GetProgDspt(order[i]);
            m_ProgList[i] = pDoc->GetProgList(order[i]);
            //szDescript.Format("%02d   %s", i, progList[order[i]].ProgDspt);
		    szDescript.Format("%02d   %s", i, m_ProgDspt[i]);
            theListCtrl.InsertItem(i, szDescript);
            theListCtrl.SetItemText(i, 1, m_ProgList[i]);
        }
        delete [] order;

		//Select the first item in the list view:
		theListCtrl.SetItemState(0, LVIS_SELECTED, LVIS_SELECTED);
        theListCtrl.EnsureVisible(0, FALSE);
    }

    m_nCurPlayIndex = 0;

    CRealTrackerApp *pApp = (CRealTrackerApp*) AfxGetApp();
    if (! pApp->IsFirstRun())
        pApp->GetUserOption(m_Pref);
    else
    {
        m_Pref.IsFirstRun = TRUE;
        PostMessage(WM_COMMAND, ID_FILE_USER);
    }
}

/////////////////////////////////////////////////////////////////////////////
// CRealTrackerView message handlers

void CRealTrackerView::OnTimer(UINT nIDEvent) 
{
    if (nIDEvent != m_nTimer)
    {
	    CListView::OnTimer(nIDEvent);
        return;
    }

    // My timer function
    KillTimer(m_nTimer);
    m_bCurRunning = FALSE;
    m_bNextRunning = TRUE;
    m_VQTest->GetRMAContext()->Stop();
    FinishPlayOne();

    m_nCurPlayIndex ++;
    if (m_nCurPlayIndex < m_nProgListNum)
    {
        StartPlayOne();
    }
    else
    {
        m_nCurPlayIndex = 0;
	    CListCtrl& theListCtrl = GetListCtrl();
        theListCtrl.SetItemState(m_nCurPlayIndex, LVIS_SELECTED, LVIS_SELECTED);
        theListCtrl.EnsureVisible(m_nCurPlayIndex, FALSE);
//        m_VideoListCtrl.EnableWindow(TRUE);
        m_bPlaying = FALSE;
        m_bNextRunning = FALSE;
        m_bCurRunning = FALSE;
    }
}

LRESULT CRealTrackerView::OnVQAction(WPARAM wParam, LPARAM lParam)
{
    UINT nID = (UINT) wParam;

	CListCtrl& theListCtrl = GetListCtrl();

    if (nID == CVQ_DATA_ON_CONTACT)
	{
//SSS		m_TimeProgress.SetPos(0);
//SSS		SetDlgItemText(IDC_STATIC_TIMELINE, "Contacting Server ...  ");
        m_ProgressLine = 0;
        m_TimeLine = "Contacting Server ...";
	}
	else if (nID == CVQ_DATA_ON_OPEN)
	{
		m_VQTest->GetRMAContext()->ReSize(m_SizeRatio);
		VQData * vq_data = (VQData*)lParam;
		CString itemStr;
//		m_FileURL.GetLBText(0, itemStr);
		vq_data->Name = new char[itemStr.GetLength()];
		strcpy(vq_data->Name, itemStr);
		m_VQTest->AddVideoStats(vq_data);
//SSS		m_TimeProgress.SetPos(0);
//SSS		SetDlgItemText(IDC_STATIC_TIMELINE, "Contacting Server ...  ");
        m_ProgressLine = 0;
        m_TimeLine = "Contacting Server ...";
	}
	else if (nID == CVQ_DATA_ON_BEGIN)
	{
        m_bOnBegined = TRUE;
    }
	else if (nID == CVQ_DATA_ON_BUFFER)
	{
		// lParm = percentage
        if (lParam >=0 && lParam <= 100)
        {
//SSS			m_TimeProgress.SetPos(lParam);
//SSS			SetDlgItemText(IDC_STATIC_TIMELINE, "On Buffering ...       ");
            m_ProgressLine = lParam;
            m_TimeLine = "On Buffering ...";
		}

		if(lParam == -2)
			m_VQTest->GetRMAContext()->Stop();
	}
	else if (nID == CVQ_DATA_ON_CLOSE)
	{
        if (m_bCurRunning)
        {
		    m_bCurRunning = FALSE;
            if (m_Pref.PlayTime != PLAYTIME_WHOLE)
                KillTimer(m_nTimer);

            FinishPlayOne();

            m_nCurPlayIndex ++;
            if (m_nCurPlayIndex < m_nProgListNum)
            {
                StartPlayOne();
                m_bCurRunning = TRUE;
                m_bNextRunning = FALSE;
            }
            else
            {
                m_nCurPlayIndex = 0;
                theListCtrl.SetItemState(m_nCurPlayIndex, LVIS_SELECTED, LVIS_SELECTED);
                theListCtrl.EnsureVisible(m_nCurPlayIndex, FALSE);
                m_bPlaying = FALSE;
//                m_VideoListCtrl.EnableWindow(TRUE);
            }

            m_ProgressLine = 0;
            m_TimeLine = "";

        }
        else if (m_bNextRunning)
        {
            m_bCurRunning = TRUE;
            m_bNextRunning = FALSE;
        }
	}
	else if (nID == CVQ_DATA_ON_STATISTICS)
	{
		VQData * vq_data;
		vq_data = (VQData*)lParam;
		m_VQTest->AddVideoStats(vq_data);
//        m_VQTest->GetMediaLog()->AddInterval(vq_data->NumOfPeriod, vq_data->intervals);
	}
	else if (nID == CVQ_DATA_ON_POSITION)
	{
		VQData * vq_data;
		vq_data = (VQData*)lParam;
		m_VQTest->ShowTimeLine(vq_data->playTime, vq_data->totalTime);

        // start the timer at the first timer after buffering
        if (m_TimerValue > 0 && vq_data->playTime > 0 && vq_data->totalTime > 0)
        {
            m_nTimer = SetTimer(0x1010, m_TimerValue, 0);
            m_TimerValue = 0;
        }
	}
	else if (nID == CVQ_DATA_ON_MINUTE)
	{
		VQJitter * vq_jitter;
		vq_jitter = (VQJitter*)lParam;
		m_VQTest->AddVideoJitter(vq_jitter);
	}

    return 0;
}

void CRealTrackerView::OnUpdateProgress(CCmdUI *pCmdUI) 
{
    pCmdUI->Enable(); 
    pCmdUI->SetText("");
    ((CTimeStatusBar *)pCmdUI->m_pOther)->SetProgressPos(m_ProgressLine);
}

void CRealTrackerView::OnUpdateTimeLine(CCmdUI *pCmdUI) 
{
    pCmdUI->Enable(); 
    pCmdUI->SetText(m_TimeLine); 
}

void CRealTrackerView::OnControlPlay() 
{
	CListCtrl& theListCtrl = GetListCtrl();

    if (theListCtrl.GetItemCount() < 1)
        return;

    POSITION pos = theListCtrl.GetFirstSelectedItemPosition();
    if (pos == NULL)
        m_nCurPlayIndex = 0;
    else
        m_nCurPlayIndex = theListCtrl.GetNextSelectedItem(pos);
//    m_VideoListCtrl.EnableWindow(FALSE);
    StartPlayOne();
    m_bPlaying = TRUE;
    m_bCurRunning = TRUE;
    m_bNextRunning = FALSE;
}

void CRealTrackerView::OnControlStop() 
{
    if (m_Pref.PlayTime != PLAYTIME_WHOLE)
        KillTimer(m_nTimer);
    m_bPlaying = FALSE;
    m_bCurRunning = FALSE;
    m_VQTest->GetRMAContext()->Stop();
    m_bNextRunning = FALSE;
//    m_VideoListCtrl.EnableWindow(TRUE);
    m_ProgressLine = 0;
    m_TimeLine = "";
}

void CRealTrackerView::OnUpdateControlPlay(CCmdUI* pCmdUI) 
{
    pCmdUI->Enable(! m_bPlaying);
}

void CRealTrackerView::OnUpdateControlStop(CCmdUI* pCmdUI) 
{
    pCmdUI->Enable(m_bPlaying);
}

void CRealTrackerView::OnControlOption() 
{
    CControlOptionDlg dlg;

    dlg.m_nPlayTime = m_Pref.PlayTime;
    dlg.m_nCheckInput = m_Pref.CheckQuality;
    if (dlg.DoModal() == IDOK)
    {
        m_Pref.PlayTime = dlg.m_nPlayTime;
        m_Pref.CheckQuality = dlg.m_nCheckInput;
        CRealTrackerApp *pApp = (CRealTrackerApp *) AfxGetApp();
        pApp->WriteUserOption(m_Pref);
    }
}


void CRealTrackerView::OnFileUser() 
{
    CUserInfoDlg dlg;

    dlg.m_bDisableCancel    = m_Pref.IsFirstRun;
    dlg.m_strName           = m_Pref.UserName;
    dlg.m_strEmailAddr      = m_Pref.EmailAddr;
    dlg.m_strCountry        = m_Pref.Country;
    dlg.m_strState          = m_Pref.State;
    dlg.m_nCpuType          = m_Pref.UserCpu;
    dlg.m_nNetConfig        = m_Pref.NetConfig;

    if (dlg.DoModal() == IDOK)
    {
        m_Pref.UserName      =    dlg.m_strName;
        m_Pref.EmailAddr     =    dlg.m_strEmailAddr;
        m_Pref.Country       =    dlg.m_strCountry;
        m_Pref.State         =    dlg.m_strState;
        m_Pref.UserCpu       =    dlg.m_nCpuType;
        m_Pref.NetConfig     =    dlg.m_nNetConfig;

        CRealTrackerApp *pApp = (CRealTrackerApp *) AfxGetApp();
        pApp->WriteUserOption(m_Pref);
    }
    m_Pref.IsFirstRun = FALSE;
}

void CRealTrackerView::OnViewStat() 
{
    if (m_StatsDlg == NULL)
    {
        m_StatsDlg = new CStatsDlg(this);
        if (! m_StatsDlg->Create(IDD_STATS, this))
        {
            MessageBox("Error creating dialog", "Error");
            return;
        }
        m_StatsDlg->SetTopView(this);
    }

    if (! m_StatsDlgIsVisable)
    {
        m_StatsDlg->ShowWindow(SW_SHOW);
        m_StatsDlg->SetFocus();
    }
    else
        m_StatsDlg->ShowWindow(SW_HIDE);

    m_StatsDlgIsVisable = ! m_StatsDlgIsVisable;
}

void CRealTrackerView::OnUpdateViewStat(CCmdUI* pCmdUI) 
{
    pCmdUI->SetCheck(m_StatsDlgIsVisable);
}

/////////////////////////////////////////////////////////////////////////////
//  All user functions
/////////////////////////////////////////////////////////////////////////////

int CRealTrackerView::GetProgListNum()     { return m_nProgListNum; }

CString *CRealTrackerView::GetProgDspt()   { return m_ProgDspt; }

CString *CRealTrackerView::GetProgList()   { return m_ProgList; }

int CRealTrackerView::GetCurPlayIndex()    { return m_nCurPlayIndex; }

void CRealTrackerView::ShowProgressBarAndTimeLine(int Prog, char *TimeLine)
{
//SSS    m_TimeProgress.SetPos(Prog);
//SSS    SetDlgItemText(IDC_STATIC_TIMELINE, TimeLine);
    m_ProgressLine = Prog;
    m_TimeLine = TimeLine;
}

CVQTest *CRealTrackerView::GetVQTest()
{
    return m_VQTest;
}

CStatsDlg *CRealTrackerView::GetStatsDlg()
{
    if (m_StatsDlgIsVisable)
        return m_StatsDlg;
    else
        return NULL;
}

void CRealTrackerView::HideStatsDlg()
{
    m_StatsDlgIsVisable = FALSE;
}

BOOL CRealTrackerView::StatsDlgIsVisable()
{
    return m_StatsDlgIsVisable;
}

/////////////////////////////////////////////////////////////////////////////////////////
void CRealTrackerView::StartPlayOne()
{
//    m_VideoListCtrl.SetCurSel(m_nCurPlayIndex);
    CListCtrl& theListCtrl = GetListCtrl();
    theListCtrl.SetItemState(m_nCurPlayIndex, LVIS_SELECTED, LVIS_SELECTED);
    theListCtrl.EnsureVisible(m_nCurPlayIndex, FALSE);

    m_VQTest->InitVideoStats(600);
    m_VQTest->GetRMAContext()->Play((LPSTR)(LPCSTR)m_ProgList[m_nCurPlayIndex]);
    m_bOnBegined = FALSE;

    time_t ltime;
    struct tm *gmt;
    time(&ltime);
    gmt = gmtime(&ltime);
    m_StartTime = asctime(gmt);
    // Remove '\n' at the end of string
    if (m_StartTime[m_StartTime.GetLength() - 1] == '\n')
        m_StartTime = m_StartTime.Left(m_StartTime.GetLength() - 1);

    m_TimerValue = 0;
    if (m_Pref.PlayTime != PLAYTIME_WHOLE)
    {
        int delayMsec;
        if (m_Pref.PlayTime == PLAYTIME_ONE)
            delayMsec = 60000;
        else if (m_Pref.PlayTime == PLAYTIME_TWO)
            delayMsec = 120000;
        else if (m_Pref.PlayTime == PLAYTIME_THREE)
#ifndef _DEBUG
            delayMsec = 180000;
#else
        {
            delayMsec = 180000;
            if (m_Pref.UserName == "mark man")
                delayMsec = 2000;
        }
#endif
        //m_nTimer = SetTimer(0x1010, delayMsec, 0);
        m_TimerValue = delayMsec;  // starting timer after buffering
    }
}

void CRealTrackerView::FinishPlayOne()
{
    time_t ltime;
    struct tm *gmt;
    time(&ltime);
    gmt = gmtime(&ltime);
    m_EndTime = asctime(gmt);
    // Remove '\n' at the end of string
    if (m_EndTime[m_EndTime.GetLength() - 1] == '\n')
        m_EndTime = m_EndTime.Left(m_EndTime.GetLength() - 1);

    GetSystemTime(&m_st); // gets current system time for file name

    BOOL bInputQuality = FALSE;
    if (m_bOnBegined)     // clip played or not (cannot access this video file on server)
    {
        if (m_Pref.CheckQuality == CHECKQUALITY_ALWAYS)
            bInputQuality = TRUE;
        else if (m_Pref.CheckQuality == CHECKQUALITY_BYCHOICE)
        {
            CCheckInputDlg chkDlg;
            if (chkDlg.DoModal() ==IDOK)
                bInputQuality = TRUE;
        }
    }

    m_nQuaValue = 999;
    if (bInputQuality)
    {
        CQualityInputDlg dlg;
        dlg.DoModal();
        m_nQuaValue = dlg.m_nQualityValue;
    }

//SetDlgItemText(IDC_EDIT_SHOWSTATS, "Mail Result --> OK");
//SetDlgItemText(IDC_EDIT_SHOWSTATS, "Mail Result --> Error");

    if (m_AdvPref.bUseMail1)
        MailResult(m_AdvPref.SMTPServer1, m_AdvPref.UserName1, m_AdvPref.UserAddr1, m_bOnBegined);
    if (m_AdvPref.bUseMail2)
        MailResult(m_AdvPref.SMTPServer2, m_AdvPref.UserName2, m_AdvPref.UserAddr2, m_bOnBegined);

    if (m_AdvPref.bUseFtp1)
        FtpResult(m_AdvPref.ftpServer1, m_AdvPref.ftpUser1, m_AdvPref.ftpPasswd1, m_AdvPref.ftpDir1, m_bOnBegined);
    if (m_AdvPref.bUseFtp2)
        FtpResult(m_AdvPref.ftpServer2, m_AdvPref.ftpUser2, m_AdvPref.ftpPasswd2, m_AdvPref.ftpDir2, m_bOnBegined);

    if (m_bSaveToFile)
        SaveResult(m_SaveToDir, m_bOnBegined);
}

/*
from smtp.wpi.edu or emcmail.emc.com
EHLO localhost
MAIL FROM: <anyuser@anydomail>
RCTP TO: <anyuser@anydomail>
DATA
Date: Tue, 15 May 2001 11:24:24 -0400
Subject: EMAIL-SUBJECT

EMAIL-Content!

.
QUIT
*/
BOOL CRealTrackerView::MailResult(CString smtpserver, CString username, CString useraddr, BOOL bPlayed)
{
    // Construct a socket from the derived class
    CSocketX theSocket;

    // Connect to the server
    if (! theSocket.ConnectToServer(smtpserver))
        return FALSE;

	// General purpose strings
	CString strCommand;
	CString strResponse;
	UINT uTimeOut = 10000; // Set timeout to 10 seconds

	// Read the connection response from the server
	if (theSocket.Receive(strResponse, uTimeOut) == SOCKET_ERROR)
		return FALSE;

	// and check to see if we're talking to an SMTP server
	if (strResponse.Left(3) != _T("220"))
	{
		theSocket.Send("QUIT\r\n");
		return FALSE;
	}

    // Send the "EHLO" line
    strCommand = "EHLO localhost\r\n";
	if (! theSocket.SendAndResponse(strCommand, uTimeOut, _T("250")))
		return FALSE;

    // Send the "FROM" line
    strCommand = "MAIL FROM: <";
    strCommand += m_Pref.EmailAddr;
    strCommand += ">\r\n";
	if (! theSocket.SendAndResponse(strCommand, uTimeOut, _T("250")))
		return FALSE;

	// Send the "RCPT" line
    strCommand = "RCPT TO: <";
    strCommand += useraddr;
    strCommand += ">\r\n";
	if (! theSocket.SendAndResponse(strCommand, uTimeOut, _T("250")))
		return FALSE;

	// Send the "DATA" line
    strCommand = "DATA\r\n";
	if (! theSocket.SendAndResponse(strCommand, uTimeOut, _T("354")))
		return FALSE;

	// No response from server expectd for following lines

    // Send the "From" line
    strCommand = "From: \"";
    strCommand += m_Pref.UserName.IsEmpty() ? "anonym" : m_Pref.UserName;
    strCommand += "\" <";
    strCommand += m_Pref.EmailAddr.IsEmpty() ? "unknown_name@unknown_domain" : m_Pref.EmailAddr;
    strCommand += ">\r\n";
	theSocket.Send(strCommand);
    // Send the "To" line
    strCommand = "To: \"";
    strCommand += username;
    strCommand += "\" <";
    strCommand += useraddr;
    strCommand += ">\r\n";
	theSocket.Send(strCommand);
    // Send the "Subject" line
    FILETIME ft;
    CString filename;
    SystemTimeToFileTime(&m_st, &ft);  // converts to file time format
    filename.Format(" %010u%010u-%03d", ft.dwHighDateTime, ft.dwLowDateTime, m_nCurPlayIndex);
    CString ipaddr = theSysStatus.GetIPAddr();
    strCommand = "Subject: RealTracer ";
    strCommand += theSysStatus.GetMachineName();
    if (! ipaddr.IsEmpty())
    {
        strCommand += "(";
        strCommand += ipaddr;
        strCommand += ")";
    }
    strCommand += filename;
    if (! bPlayed)
        strCommand += " - FILE NOT FOUND !!!";
    strCommand += "\r\n";
	theSocket.Send(strCommand);

	// Send the log data
	if (! OutputLog(&theSocket, NULL, NULL, bPlayed))
		return FALSE;

	// Send the termination line
    strCommand = "\r\n.\r\n";
	if (! theSocket.SendAndResponse(strCommand, uTimeOut, _T("250")))
		return FALSE;

	// Send the "QUIT" line
	theSocket.Send("QUIT\r\n");

	return TRUE;
}

BOOL CRealTrackerView::FtpResult(CString server, CString user, CString passwd, CString dir, BOOL bPlayed)
{
    // use the time as the file name
    CString filename;
    FILETIME ft;

    CFtpSave ftp(server, user, passwd);
    if (! ftp.bConnected())
        return FALSE;
//    CString dirName = "RealTracer/";
    CString dirName = dir + "/";
    dirName += theSysStatus.GetMachineName();
    ftp.Makedir(dirName);
    SystemTimeToFileTime(&m_st, &ft);  // converts to file time format
    filename.Format("/%010u%010u-%03d", ft.dwHighDateTime, ft.dwLowDateTime, m_nCurPlayIndex);
    if (! bPlayed)
        filename += "__FILE_NOT_FOUND";
    CString ftpfile = dirName + filename;
    ftp.OpenFile(ftpfile);
    if (! ftp.bFileOpened())
        return FALSE;

	// Ftp the log data
	if (! OutputLog(NULL, &ftp, NULL, bPlayed))
		return FALSE;

    ftp.CloseFile();
    ftp.CloseConnection();

    return TRUE;
}

BOOL CRealTrackerView::SaveResult(CString dir, BOOL bPlayed)
{
    // use the time as the file name
    CString filename;
    FILETIME ft;

    SystemTimeToFileTime(&m_st, &ft);  // converts to file time format
    filename.Format("/%010u%010u-%03d", ft.dwHighDateTime, ft.dwLowDateTime, m_nCurPlayIndex);
    if (! bPlayed)
        filename += "__FILE_NOT_FOUND";
    CString filepath = dir + filename;

    FILE *fp = fopen((LPCSTR)filepath, "w");
    if (fp == NULL)
        return FALSE;

	// Ftp the log data
	if (! OutputLog(NULL, NULL, fp, bPlayed))
    {
        fclose(fp);
		return FALSE;
    }

    fclose(fp);

    return TRUE;
}

// EMail or Ftp all log data
BOOL CRealTrackerView::OutputLog(CSocketX *pSocket, CFtpSave *pFtp, FILE *fp, BOOL bPlayed)
{
    if (! OutputUserInfo(pSocket, pFtp, fp))
        return FALSE;
    if (bPlayed)
    {
        if (! OutputResultData(pSocket, pFtp, fp))
            return FALSE;
        if (! OutputVideoQuality(pSocket, pFtp, fp))
            return FALSE;
    }
    else
    {
        CString szMessage = "FILE NOT FOUND !!!\r\n";
        if (! OutputOneLine(pSocket, pFtp, fp, szMessage))
            return FALSE;
    }
    return TRUE;
}

// EMail or Ftp user information
BOOL CRealTrackerView::OutputUserInfo(CSocketX *pSocket, CFtpSave *pFtp, FILE *fp)
{
    CString szMessage;
    char *pStr;

    if (! m_Pref.UserName.IsEmpty())
        szMessage = "User Name: " + m_Pref.UserName;
    else
        szMessage = "User Name: anonym";
    szMessage += "\r\n";
    if (! OutputOneLine(pSocket, pFtp, fp, szMessage))
        return FALSE;

    if (! m_Pref.EmailAddr.IsEmpty())
        szMessage = "User E-Mail Address: " + m_Pref.EmailAddr;
    else
        szMessage = "User E-Mail Address: unknown_name@unknown_domain";
    szMessage += "\r\n";
    if (! OutputOneLine(pSocket, pFtp, fp, szMessage))
        return FALSE;

    szMessage = "User Location (Country): " + m_Pref.Country;
    szMessage += "\r\n";
    if (! OutputOneLine(pSocket, pFtp, fp, szMessage))
        return FALSE;

    szMessage = "User Location (State/Province): " + m_Pref.State;
    szMessage += "\r\n";
    if (! OutputOneLine(pSocket, pFtp, fp, szMessage))
        return FALSE;

    szMessage = "IP Address: " + theSysStatus.GetIPAddr();
    szMessage += "\r\n";
    if (! OutputOneLine(pSocket, pFtp, fp, szMessage))
        return FALSE;

    szMessage = "CPU (auto-detected): " + m_Pref.CPUType;
    szMessage += "\r\n";
    if (! OutputOneLine(pSocket, pFtp, fp, szMessage))
        return FALSE;

    szMessage.Format("CPU (user-defined): %d\r\n", m_Pref.UserCpu);
    if (! OutputOneLine(pSocket, pFtp, fp, szMessage))
        return FALSE;

    szMessage.Format("Memory(M): %d\r\n", m_Pref.MEMCap);
    if (! OutputOneLine(pSocket, pFtp, fp, szMessage))
        return FALSE;

    szMessage.Format("OS Type: %d\r\n", m_Pref.OSType);
    if (! OutputOneLine(pSocket, pFtp, fp, szMessage))
        return FALSE;

    szMessage.Format("Network Configuration: %d\r\n", m_Pref.NetConfig);
    if (! OutputOneLine(pSocket, pFtp, fp, szMessage))
        return FALSE;

    if (m_Pref.PlayTime == PLAYTIME_ONE)
        szMessage = "Play Time: One Minute\r\n";
    else if (m_Pref.PlayTime == PLAYTIME_TWO)
        szMessage = "Play Time: Two Minutes\r\n";
    else if (m_Pref.PlayTime == PLAYTIME_THREE)
        szMessage = "Play Time: Three Minutes\r\n";
    else if (m_Pref.PlayTime == PLAYTIME_WHOLE)
        szMessage = "Play Time: Whole clip";
    if (! OutputOneLine(pSocket, pFtp, fp, szMessage))
        return FALSE;

    szMessage = "\r\n";
    if (! OutputOneLine(pSocket, pFtp, fp, szMessage))
        return FALSE;

    szMessage = "Start time: " + m_StartTime;
    szMessage += "\r\n";
    if (! OutputOneLine(pSocket, pFtp, fp, szMessage))
        return FALSE;

    szMessage = "End time: " + m_EndTime;
    szMessage += "\r\n";
    if (! OutputOneLine(pSocket, pFtp, fp, szMessage))
        return FALSE;

    szMessage = "\r\n";
    if (! OutputOneLine(pSocket, pFtp, fp, szMessage))
        return FALSE;

    szMessage.Format("Video %02d - %s - %s\r\n\r\n", m_nCurPlayIndex, m_ProgDspt[m_nCurPlayIndex], m_ProgList[m_nCurPlayIndex]);
    if (! OutputOneLine(pSocket, pFtp, fp, szMessage))
        return FALSE;

    CMediaLog *pMediaLog = m_VQTest->GetMediaLog();
    pStr = pMediaLog->GetServerInfo();
    szMessage.Format("Server: %s\r\n", (pStr ? pStr : "Unknown"));
    if (! OutputOneLine(pSocket, pFtp, fp, szMessage))
        return FALSE;

    pStr = pMediaLog->GetVideoCodecName();
    szMessage.Format("Codec: %s\r\n\r\n", (pStr ? pStr : "Unknown"));
    if (! OutputOneLine(pSocket, pFtp, fp, szMessage))
        return FALSE;

    return TRUE;
}

// EMail or Ftp result data
BOOL CRealTrackerView::OutputResultData(CSocketX *pSocket, CFtpSave *pFtp, FILE *fp)
{
    CString szMessage;
    CMediaStatistics *pVideoStats = m_VQTest->GetMediaLog()->GetVideoStats();
    VQDataLink *pLink;
    VQData *pData = pVideoStats->GetVQDataFirst(pLink);

    szMessage = "CodedBw AvgBw TotalBw CurBw AvgLtcy Transport CodedFRate AvgFRate CurFRate FDrop FLost AvgJit CurJit MaxJit CpuUsage\r\n";
    if (! OutputOneLine(pSocket, pFtp, fp, szMessage))
        return FALSE;

    while (pLink)
    {
        szMessage.Format("%d %d %d %d %d %d %.2lf %.2lf %.2lf %d %d %.2lf %.2lf %.2lf %.2lf\r\n", 
                    pData->CodedBandWidth,
                    pData->AverageBandWidth,
                    pData->TotalBandWidth,
                    pData->CurrentBandWidth,
                    pData->AverageLatency,
                    pData->TransportMode,
                    pData->CodedFrameRate,
                    pData->AverageFrameRate,
                    pData->CurrentFrameRate,
                    pData->FramesDropped,
                    pData->FramesLost,
                    pData->AverageJitter,
                    pData->CurrentJitter,
                    pData->MaxJitter,
                    pData->CpuUsage);
        if (! OutputOneLine(pSocket, pFtp, fp, szMessage))
            return FALSE;
        pData = pVideoStats->GetVQDataNext(pLink);
    }

    szMessage = "CodedBw AvgBw TotalBw CurBw AvgLtcy Transport CodedFRate AvgFRate CurFRate FDrop FLost AvgJit CurJit MaxJit CpuUsage\r\n\r\n";
    if (! OutputOneLine(pSocket, pFtp, fp, szMessage))
        return FALSE;

    return TRUE;
}

// EMail or Ftp video quality
BOOL CRealTrackerView::OutputVideoQuality(CSocketX *pSocket, CFtpSave *pFtp, FILE *fp)
{
    CString szMessage;

    if (m_nQuaValue == 999)
        szMessage = "Quality: Not Assessed\r\n";
    else
        szMessage.Format("Quality: %d\r\n", m_nQuaValue);
    return OutputOneLine(pSocket, pFtp, fp, szMessage);
}

// EMail or Ftp one line message
BOOL CRealTrackerView::OutputOneLine(CSocketX *pSocket, CFtpSave *pFtp, FILE *fp, CString szMessage)
{
    if (pSocket != NULL)
    {
        if (pSocket->Send(szMessage) == SOCKET_ERROR)
            return FALSE;
    }
    else if (pFtp != NULL)
    {
        pFtp->Write(szMessage, strlen(szMessage));
        if (! pFtp->bFileOpened())
            return FALSE;
    }
    else
    {
        // remove '\r' in the szMessage
        szMessage.Remove('\r');
        fprintf(fp, "%s", (LPCSTR) szMessage);
    }
    return TRUE;
}

/*void CRealTrackerView::OnRButtonUp(UINT nFlags, CPoint point) 
{
    if ((nFlags & MK_CONTROL) && (nFlags & MK_SHIFT))
    {
        RECT rect1, rect2, rect3;

        GetWindowRect(&rect1);
        GetDlgItem(IDC_BUTTON_OPTION)->GetWindowRect(&rect2);
        GetDlgItem(IDC_BUTTON_EXIT)->GetWindowRect(&rect3);
        int left = rect2.right - rect1.left - 4;
        int right = rect3.left - rect1.left - 4;
        int top = rect2.top - rect1.top - 22;
        int bottom = rect2.bottom - rect1.top - 22;
        int midx = (left + right) / 2;
        if (left < point.x && point.x < midx && top < point.y && point.y < bottom)
        {
            CAdvPreferenceDlg dlg;

            dlg.m_bUseMailWpi   = m_AdvPref.bUseWpiMail;
            dlg.m_strMailName1  = m_AdvPref.wpiUserName;
            dlg.m_strMailAddr1  = m_AdvPref.wpiUserAddr;

            dlg.m_bUseMailOther = m_AdvPref.bUseEmcMail;
            dlg.m_strMailName2  = m_AdvPref.emcUserName;
            dlg.m_strMailAddr2  = m_AdvPref.emcUserAddr;

            dlg.m_bUseFtp1      = m_AdvPref.bUseFtp1;
            dlg.m_strFtpServer1 = m_AdvPref.ftpServer1;
            dlg.m_strFtpUser1   = m_AdvPref.ftpUser1;
            dlg.m_strFtpPasswd1 = m_AdvPref.ftpPasswd1;

            dlg.m_bUseFtp2      = m_AdvPref.bUseFtp2;
            dlg.m_strFtpServer2 = m_AdvPref.ftpServer2;
            dlg.m_strFtpUser2   = m_AdvPref.ftpUser2;
            dlg.m_strFtpPasswd2 = m_AdvPref.ftpPasswd2;


            if (dlg.DoModal() == IDOK)
            {
                m_AdvPref.bUseWpiMail = dlg.m_bUseMailWpi;
                m_AdvPref.wpiUserName = dlg.m_strMailName1;
                m_AdvPref.wpiUserAddr = dlg.m_strMailAddr1;
                                                           
                m_AdvPref.bUseEmcMail = dlg.m_bUseMailOther;
                m_AdvPref.emcUserName = dlg.m_strMailName2;
                m_AdvPref.emcUserAddr = dlg.m_strMailAddr2;
                                                           
                m_AdvPref.bUseFtp1    = dlg.m_bUseFtp1;
                m_AdvPref.ftpServer1  = dlg.m_strFtpServer1;
                m_AdvPref.ftpUser1    = dlg.m_strFtpUser1;
                m_AdvPref.ftpPasswd1  = dlg.m_strFtpPasswd1;
                                                           
                m_AdvPref.bUseFtp2    = dlg.m_bUseFtp2;
                m_AdvPref.ftpServer2  = dlg.m_strFtpServer2;
                m_AdvPref.ftpUser2    = dlg.m_strFtpUser2;
                m_AdvPref.ftpPasswd2  = dlg.m_strFtpPasswd2;
            }
        }
        else if (midx < point.x && point.x < right && top < point.y && point.y < bottom)
        {
            RunOnButtonStats();
        }
    }

    CDialog::OnRButtonUp(nFlags, point);
}*/

#define _A_MULTIPLIER  16807L
#define _M_MODULUS     2147483647L /* (2**31)-1 */
#define _Q_QUOTIENT    127773L     /* 2147483647 / 16807 */
#define _R_REMAINDER   2836L       /* 2147483647 % 16807 */

void MakeRandomList(int order[], int n)
{
    for (int i=0; i<n; i++)
        order[i] = i;

//#ifndef _DEBUG
    int j, k, k1;
    long seedi, lo, hi, test;

    seedi = 12345;
    for (i=n; i>1; i--)
    {
        k = seedi % i;
        k1 = order[k];
        for (j=k+1; j<i; j++)
            order[j-1] = order[j];
        order[i-1] = k1;

        hi = seedi / _Q_QUOTIENT;
        lo = seedi % _Q_QUOTIENT;
        test = _A_MULTIPLIER * lo - _R_REMAINDER * hi;
        seedi = test > 0 ? test : test + _M_MODULUS;
    }
//#endif
}

void CRealTrackerView::OnFileSavelog() 
{
    CSaveLogDlg	dlg;
    dlg.m_bSaveLog = m_bSaveToFile;
    dlg.m_DirName = m_SaveToDir;
    if (dlg.DoModal() == IDOK)
    {
        m_bSaveToFile = dlg.m_bSaveLog;
        m_SaveToDir = dlg.m_DirName;
    }
}
