/**************************************************************************
**
** Copyright (C) Computer Science Department, Worcester Polytechnic Institute,
**                           All rights reserved.
**
** FILE NAME: SysStatus.cpp
**
** DESCRIPTION: This program implements the methods of CSysStatus class
**              
**
** AUTHOR: Zheng Zuo/Yubing Wang       March-2001
**
/**************************************************************************/


// SysStatus.cpp: implementation of the CSysStatus class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "RealTracker.h"
#include "SysStatus.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

#define MAXPATH     256

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CSysStatus::CSysStatus()
{
    TCHAR machineName[MAX_COMPUTERNAME_LENGTH + 1];
    DWORD lnSize = MAX_COMPUTERNAME_LENGTH + 1;
    if (GetComputerName(machineName, &lnSize))
        strcpy(m_strMachineName, machineName);
    else
        strcpy(m_strMachineName, "Unknown");

    m_hDll = NULL;
    int ostype = GetOSType();
    if (ostype < OSTYPE_WINDOWS9598  || ostype > OSTYPE_WINDOWS2000)
        m_bPdhSupport = 0;
    else if (ostype == OSTYPE_WINDOWS9598)
    {
        m_bPdhSupport = 1;
        m_hDll = LoadLibrary("Aux98.dll");
        if (m_hDll == NULL)
            m_bPdhSupport = 0;
        else
        {
            m_fpGetCpuLoad98 = (FPGETCPULOAD98) GetProcAddress(m_hDll, GETCPULOAD98_NAME);
            if (m_fpGetCpuLoad98 == NULL)
            {
                FreeLibrary(m_hDll);
                m_bPdhSupport = 0;
            }
        }
    }
    else
    {
        m_bPdhSupport = 2;
        m_hDll = LoadLibrary("AuxNT.dll");
        if (m_hDll == NULL)
            m_bPdhSupport = 0;
        else
        {
            m_fpPdhOpenQuery = (FPDHOPENQUERY) GetProcAddress(m_hDll, PDHOPENQUERY_NAME);
            m_fpPdhCloseQuery = (FPDHCLOSEQUERY) GetProcAddress(m_hDll, PDHCLOSEQUERY_NAME);
            m_fpPdhAddCounter = (FPDHADDCOUNTER) GetProcAddress(m_hDll, PDHADDCOUNTER_NAME);
            m_fpPdhCollectQueryData = (FPDHCOLLECTQUERYDATA) GetProcAddress(m_hDll, PDHCOLLECTQUERYDATA_NAME);
            m_fpPdhGetFormattedCounterValue = (FPDHGETFORMATTEDCOUNTERVALUE) GetProcAddress(m_hDll, PDHGETFORMATTEDCOUNTERVALUE_NAME);
            if (m_fpPdhOpenQuery == NULL || m_fpPdhCloseQuery == NULL || m_fpPdhAddCounter == NULL ||
                m_fpPdhCollectQueryData == NULL || m_fpPdhGetFormattedCounterValue == NULL)
            {
                FreeLibrary(m_hDll);
                m_bPdhSupport = 0;
            }
            else
            {
                CHAR cpu[MAXPATH];
                PDH_STATUS pdhStatus;

                // Open the query object.
                sprintf(cpu, "\\\\%s\\Processor(_Total)\\%% Processor Time", machineName);
                pdhStatus = m_fpPdhOpenQuery(0, 0, &m_hQuery_cpu);
                pdhStatus = m_fpPdhAddCounter(m_hQuery_cpu, cpu, 0, &m_CounterHandle);
            }
        }
    }


    // Get the ip address of this machine
//    if (WSAStartup(0x202, &wsaData) == SOCKET_ERROR)
//	{
//		fprintf(stderr, "WSAStartup failed with error %d\n", WSAGetLastError());
//		WSACleanup();
//		return;
//	}

	struct hostent *hp = gethostbyname(machineName);
    if (hp == NULL)
	{
//		WSACleanup();
		m_strIPAddr[0] = 0;
	}
    else
    {
        unsigned char *ip = (unsigned char *) hp->h_addr;
        if (ip == NULL)
            m_strIPAddr[0] = 0;
        else
            sprintf(m_strIPAddr, "%u.%u.%u.%u", (int)ip[0], (int)ip[1], (int)ip[2], (int)ip[3]);
    }
//    WSACleanup();
}

CSysStatus::~CSysStatus()
{
    if (m_bPdhSupport == 2)
        m_fpPdhCloseQuery(m_hQuery_cpu);

    if (m_bPdhSupport != 0 && m_hDll != NULL)
        FreeLibrary(m_hDll);
}

CString CSysStatus::GetCpuType()
{
    SYSTEM_INFO SystemInfo;
    CString processor;

    GetSystemInfo(&SystemInfo);
    if(SystemInfo.wProcessorArchitecture == PROCESSOR_ARCHITECTURE_INTEL)
    {
        switch(SystemInfo.wProcessorLevel)
        {
            case 3: processor = "Intel 80386"; break;
            case 4: processor = "Intel 80486"; break;
            case 5: processor = "Intel Pentium"; break;
            case 6: processor = "Intel Pentium Pro or Pentium II or Above"; break;
        }
    }
    else if(SystemInfo.wProcessorArchitecture == PROCESSOR_ARCHITECTURE_MIPS)
    {
        if (SystemInfo.wProcessorLevel == 4)
            processor = "MIPS R4000";
    }
    else if(SystemInfo.wProcessorArchitecture == PROCESSOR_ARCHITECTURE_ALPHA)
    {
        processor.Format("Alpha %d", SystemInfo.wProcessorLevel);
    }
    else if(SystemInfo.wProcessorArchitecture == PROCESSOR_ARCHITECTURE_PPC)
    {
        if (SystemInfo.wProcessorLevel < 6)
            processor.Format("PPC 60 %d", SystemInfo.wProcessorLevel);
        else if (SystemInfo.wProcessorLevel == 6)
            processor = "PPC 603+";
        else if (SystemInfo.wProcessorLevel == 9)
            processor = "PPC 604+";
        else if (SystemInfo.wProcessorLevel == 20)
            processor = "PPC 620";
    }
    else
        processor = "Unknown";

    CString cpu;
    cpu.Format("%d processor(s) - %s", SystemInfo.dwNumberOfProcessors, processor);

    return cpu;
}

int CSysStatus::GetMemory()
{
    MEMORYSTATUS Buffer;

    GlobalMemoryStatus(&Buffer);
    int K = Buffer.dwTotalPhys / 1024;  // KBytes
    int M8 = K / 8192;                  // 8MBytes
    if (K % 8192)
        M8 ++;
    return M8 * 8; // (Mbytes)
}

int CSysStatus::GetOSType()
{
    OSVERSIONINFO VersionInfo;

    VersionInfo.dwOSVersionInfoSize = sizeof(OSVERSIONINFO);
    GetVersionEx(&VersionInfo);

    // Get the build number for Windows NT/Windows 2000 or Win32s.

    int ret;
    if (VersionInfo.dwMinorVersion == 0)                // Windows NT/2000
    {
        if (VersionInfo.dwMajorVersion == 3)
            ret = OSTYPE_WINDOWSNT35;                   // NT3.5
        else if (VersionInfo.dwMajorVersion == 4)
            ret = OSTYPE_WINDOWSNT4;                    // NT4
        else if (VersionInfo.dwMajorVersion == 5)
            ret = OSTYPE_WINDOWS2000;                   // 2000
        else
            ret = OSTYPE_OTHER;                         // Other
    }
    else if (VersionInfo.dwMajorVersion == 4)        
        ret = OSTYPE_WINDOWS9598;   // Windows 95/98 -- No build number
    else
        ret = OSTYPE_OTHER;         // Win32s
    return ret;
}

CString CSysStatus::GetMachineName()
{
    return CString(m_strMachineName);
}

CString CSysStatus::GetIPAddr()
{
    return CString(m_strIPAddr);
}

double CSysStatus::GetCpuUsage()
{
    if (m_bPdhSupport == 0)
        return 0.0;

    if (m_bPdhSupport == 1)
    {
        long regKey = 285674124L;
        return (double) m_fpGetCpuLoad98(&regKey);
    }
    else
    {
        PDH_STATUS pdhStatus;
        PDH_FMT_COUNTERVALUE fmtValue_cpu;
        DWORD ctrType;

        // Get the current value of this counter.
        pdhStatus = m_fpPdhCollectQueryData(m_hQuery_cpu);
        pdhStatus = m_fpPdhGetFormattedCounterValue(m_CounterHandle, PDH_FMT_DOUBLE, &ctrType, &fmtValue_cpu);
        if (pdhStatus != ERROR_SUCCESS)
            return 0; 
        return fmtValue_cpu.doubleValue;
    }
}

