/****************************************************************************
 * 
 *  $Id: main.cpp,v 1.3 2000/05/19 22:57:11 ahamel Exp $
 *
 *  Copyright (C) 1995,1996,1997 RealNetworks, Inc.
 *  All rights reserved.
 *
 *  http://www.real.com/devzone
 *
 *  This program contains proprietary information of RealNetworks, Inc.,
 *  and is licensed subject to restrictions on use and distribution.
 *
 *  main.cpp
 *
 *  Example client for RealMedia Architecture (RMA).
 *
 */

/****************************************************************************
 * 
 *  Example client:
 *  
 *  This is an example client running without a graphical user interface.
 *
 *  It could be used as:
 *    - Sample source to develop a customized RMA client.
 *    - A testing tool for plugins.
 *
 */ 
//class TestFrame;

/****************************************************************************
 * Defines
 */
#define INITGUID     /* Interface ID's */

/****************************************************************************
 * Includes
 */
#ifdef _UNIX
  #include <stdlib.h> // for getenv()
#endif

#include "pntypes.h"
#include "pnwintyp.h"
#include "pncom.h"
#include "rmapckts.h"
#include "rmacomm.h"
#include "rmamon.h"
#include "rmafiles.h"
#include "rmaengin.h"
#include "rmacore.h"
#include "rmargn.h"
#include "rmaclsnk.h"
#include "rmaerror.h"
#include "rmaauth.h"
#include "rmawin.h"

#include "rmasite2.h"
#include "rmavsurf.h"

#include "os.h"
#include "fivemmap.h"
#include "exadvsnk.h"
#include "exerror.h"
#include "exsitsup.h"
#include "exaumgr.h"
#include "excontxt.h"
#include "fivemlist.h"
#include "rmaupgrd.h"
#include "exnwsite.h"
#include "exvsurf.h"
#include "rmargn.h"
#include "rmarendr.h"

#include "testframe.h"
#include "string.h"


//typedef PN_RESULT (PNEXPORT_PTR FPRMSETDLLACCESSPATH) (const char*);

 

TestFrame::TestFrame(CVQTest *pVQTest)
{
	frame_main();
}//end of constructor

TestFrame::~TestFrame(void)
{
	cleanup();
}//end of deconstructor

////////////////////////////////////////////////////////////////
void TestFrame::cleanup()
{
    if (exContext)
    {
        exContext->Release();
        exContext = NULL;
    }

    if (pErrorSinkControl)
    {
        pErrorSinkControl->RemoveErrorSink(pErrorSink);
        pErrorSinkControl->Release();
        pErrorSinkControl = NULL;
    }

    if (pPlayer)
    {
        pEngine->ClosePlayer(pPlayer);
        pPlayer->Release();
        pPlayer = NULL;
    }

    if (pEngine)
    {
        m_fpCloseEngine(pEngine);
        pEngine = NULL;
    }

    if (hDll)
    {
        FreeLibrary(hDll);
        hDll = NULL;
    }
} //end of cleanup()

int TestFrame::play_stream(char* pszURL)
{
	if (PNR_OK == pPlayer->OpenURL(pszURL))
    {
    	if (PNR_OK == pPlayer->Begin())
            return 0;
        else
            return 1;
    }
	else
		return 1;
	return 0;
} //end of play_stream();

/****************************************************************************
 *  frame_main
 *
 */
int TestFrame::frame_main()
{
    //FPRMCREATEENGINE	m_fpCreateEngine;
	//FPRMCLOSEENGINE	 	m_fpCloseEngine;
	PN_RESULT					theErr				= PNR_OK;
    char*						rphome              = NULL;

    if (!(exContext = new ExampleClientContext()))
    {
        theErr = PNR_UNEXPECTED;
        cleanup();
    }
    exContext->AddRef();		

	//CreateInstance(IID_IRMARTPPacket, rtpPacket);
    // initialize the globals
    m_fpCreateEngine	= NULL;
    m_fpCloseEngine	= NULL;
    
    // prepare/load the RMACore module
    char   szDllName[_MAX_PATH];

    // get location of rmacore from windows registry 
    DWORD bufSize;
    HKEY hKey; 
    PN_RESULT hRes;
    
    szDllName[0] = '\0'; 
    bufSize = sizeof(szDllName) - 1;

    if(ERROR_SUCCESS == RegOpenKey(HKEY_CLASSES_ROOT, "Software\\RealNetworks\\Preferences\\DT_Common", &hKey)) 
    { 
        // get the path to pnen 
        hRes = RegQueryValue(hKey, "", szDllName, (long *)&bufSize); 
        RegCloseKey(hKey); 
    }

    strcat(szDllName, "pnen3260.dll");
    
    if (!(hDll = LoadLibrary(szDllName))) 
    {
		theErr = PNR_FAILED;
        //fprintf(stdout, "Failed to load the '%s' library.\n", szDllName);
		cleanup();
    }
    //fprintf(stdout, "Loaded rmacore\n");
   
    // retrieve the proc addresses from the module
    m_fpCreateEngine = (FPRMCREATEENGINE) GetProcAddress(hDll, "CreateEngine");
    m_fpCloseEngine  = (FPRMCLOSEENGINE)  GetProcAddress(hDll, "CloseEngine");
 
  
    if (m_fpCreateEngine == NULL ||	m_fpCloseEngine == NULL)
    {
        theErr = PNR_FAILED;
        cleanup();
    }

    // create client engine 
    if (PNR_OK != m_fpCreateEngine((IRMAClientEngine**) &pEngine))
    {
        theErr = PNR_FAILED;
        cleanup();
    }
	
    // create player
    if (PNR_OK != pEngine->CreatePlayer(pPlayer))
    {
        theErr = PNR_FAILED;
        cleanup();
    }

    // initialize the example context
    exContext->Init(pPlayer);

    pPlayer->SetClientContext(exContext);

	pPlayer->QueryInterface(  IID_IRMAErrorSinkControl, (void**) &pErrorSinkControl);
    if ( pErrorSinkControl )
    {	
		exContext->QueryInterface(  IID_IRMAErrorSink, (void**) &pErrorSink);
		if (pErrorSink)
			pErrorSinkControl->AddErrorSink(pErrorSink, PNLOG_EMERG, PNLOG_INFO);
    }	

	return theErr;
}//end of frame_main()

void TestFrame::stop_play()
{
	if(pPlayer != NULL)
	{
		pPlayer->Stop();
		cleanup();
		pPlayer = NULL;
	}
}//end of stop_play()

void TestFrame::setWindowSize(double SizeRatio)
{
	UINT32 any=0, uRequestID=0;
	exContext->m_pSiteSupplier->ChangeWindoweSize(SizeRatio);
	//uRequestID = exContext->m_pSiteSupplier->GetRequestId(any);
	//exContext->m_pSiteSupplier->SitesNotNeeded(ID);
	//exContext->m_pSiteSupplier->getSite(uRequestID, pSite);
	//PNxSize	size;
	//pSite->GetSize(size);
	//pSite->SetSize(size);
	//exContext->m_pSiteSupplier->ChangeWindowe();
}//end of setWindowSize()
