/**************************************************************************
**
** Copyright (C) Computer Science Department, Worcester Polytechnic Institute,
**                           All rights reserved.
**
** FILE NAME: RealTraceData.h
**
** DESCRIPTION: This class defines a set of data strutures that contain the
**              the user and system information and RealVideo statistics
**
** AUTHOR: Zheng Zuo/Yubing Wang        March-2001
**
/**************************************************************************/


// RealTraceData.h: interface for the CRealTraceData class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_REALTRACEDATA_H__3871A076_9C3A_47FB_8C14_5E38ADFA5A95__INCLUDED_)
#define AFX_REALTRACEDATA_H__3871A076_9C3A_47FB_8C14_5E38ADFA5A95__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#define SZCPUTYPE_LENGTH            20
const char szCPUType[][SZCPUTYPE_LENGTH] = 
{
    "Not Specified",
    "Intel Pentium MMX",
    "Intel Celeron",
    "Intel Celeron II",
    "Intel Pentium II",
    "Intel Pentium III",
    "Intel Pentium 4",
    "Intel Pentium Xeron",
    "Intel Xeron",
    "Intel Other",
    "AMD K6",
    "AMD k6-2",
    "AMD k6-2+",
    "AMD K6-3",
    "AMD K6-3+",
    "AMD Duron",
    "AMD Athlon",
    "AMD Athlon 4",
    "AMD Other",
    "Cyrix GX",
    "Cyrix MX",
    "Cyrix M II",
    "Cyrix VIA III",
    "Cyrix VIA C3",
    "Cyrix Other",
};

#define SZOSTYPE_LENGTH             16
const char szOSType[][SZOSTYPE_LENGTH] = 
{
    "Windows 95/98",
    "Windows NT 3.5",
    "Windows NT 4",
    "Windows 2000",
    "OTHER",
};

#define SZNETCONFIG_LENGTH          20
const char szNetConfig[][SZNETCONFIG_LENGTH] = 
{
    "Not Specified",
    "14.4K Modem",
    "19.2K Connection",
    "28.8K Modem",
    "33.6K Modem",
    "56K Modem",
    "112K Dual ISDN",
    "DSL/Cable",
    "T1 / LAN",
};

#define SZTRANSPROTOCOL_LENGTH      16
const char szTransProtocol[][SZTRANSPROTOCOL_LENGTH] = 
{
    "Unknown",
    "TCP",
    "UDP",
    "RTSPvHTTP",
    "HTTP",
    "Unknown",
};

#define SZCOLUMNHEAD_LENGTH     16
const char szColumnHead[][SZCOLUMNHEAD_LENGTH] = 
{
    "FileName",
    "Location",
    "IPAddress",
    "CPUType",
    "Memory",
    "OSType",
    "NetConfig",
    "Time",
    "VideoIndex",
    "VideoName",
    "RealServerInfo",
    "RealCodec",
    "TransProtocol",
    "AvgBandWidth",
    "AvgFrameRate",
    "AvgJitter",
    "AvgCpuUsage",
    "Quality",
    "Coded FR",
	"Coded BW",
	"Frame Lost",
    "Data Number",
};

class CRealData : public CObject
{
public:
    CString FileName;
    CString Location;     // Country/State
    CString IPAddress;
    int CPUType;
    int Memory;
    int OSType;
    int NetConfig;
//	int Protocol;
//    CString Time;
    int Time;             // seconds from midnight(00:00:00)
    int VideoIndex;
    CString VideoName;
    CString RealServerInfo;
    CString RealCodec;
    CString VSize;
    int TransProtocol;
	int CodedFR;
	int CodedBW;
	int FrameDropped;
	int FrameLost;
    double AvgBandWidth;
    double AvgFrameRate;
    double AvgJitter;
    double AvgCpuUsage;
    double Quality;
    int DataNum;
};


class CRealTraceData  
{
private:
    int m_nItemNum;
    int m_nMaxItem;
    CString *m_szItemName;
    BOOL *m_bItemRead;
    int *m_nSubItemNum;
    CStringArray *m_szFilePath;
    CPtrArray *m_pRealData;

public:
	CRealTraceData();
	virtual ~CRealTraceData();

    int GetItemNum();
    CString GetItemName(int index);
    int GetSubItemNum(int index);
    CString GetSubItemPath(int index, int subIndex);
    CRealData *GetSubItemData(int index, int subIndex);

    void AddItem(CString szItemName);
    void AddSubItem(int index, CString filePath);
    void DeleteSubItem(int index, int subIndex);

private:
    void ReadRootDir();
    void ReadSubDir(int index);
    void ReadTracerFile(CString filename, CRealData &data);
};

#endif // !defined(AFX_REALTRACEDATA_H__3871A076_9C3A_47FB_8C14_5E38ADFA5A95__INCLUDED_)
